#' gets dimensions of image, x, y, channs
#' @param img image to get dimensions of
#' @return vector (nx, ny, numchans)
#' @export
imgetdims <- function(img)
{
    return( c(imgetnx(img,1), imgetny(img,1), imgetnumcomp(img)) )
}

#' subsample an image
#' 
#' Just a wrapper around \code{\link[Voir]{imdownsample}}.
#' @param src  input image to downsample
#' @param dx   dx skip
#' @param dy   dy skip
#' @param method "every" or "mean". "every" takes every dx'th and dy'th pixel
#' to form the subsampled image, whilst "mean" uses the mean in every 
#' non-overlappingdx by dy region.
#' @note It's basically because I can never remember which of 0 and 1 is 
#' every or mean when using \code{\link[Voir]{imdownsample}}.
#' @seealso \code{\link[Voir]{imdownsample}}
#' @export
imsubsample <- function(src,dx,dy,method=c('every','mean')) {
    type <- match.arg(method,c('every','mean'))
    type <- switch(type,
                   every=0,
                   mean=1
                   )
    return( imdownsample(src,dx,dy,type) )
}

#' pastes patch image into src image starting at (src) fx,fy
#' @param src source image
#' @param patch image to paste into src
#' @param fx start x/row of src to paste patch into.
#' @param fy start y/col of src to paste patch into.
#' @param one.based whether fx and fy are one-based or 0-based.
#' @return the image src with patch pasted in at fx, fy
#' @export
impaste <- function( patch, src, fx, fy, one.based=TRUE )
{
    if ( one.based ) {
        fx <- fx - 1
        fy <- fy - 1
    }
    patchI <- imchar(patch)
    imsetfirstx(patchI,IM.ALL,fx)
    imsetfirsty(patchI,IM.ALL,fy)
    patchImageOut <- impatchwithimage(src,patchI)
    return(patchImageOut)
    #return(impatchwithimage(src,patchI))
}

#' Produces a subset of an image
#' @param img input image to subset
#' @param fx  first x/col
#' @param fy  first y/row
#' @param lx  last x/col
#' @param ly  last y/row
#' @param one.based boolean, default TRUE, whether fx, fy, lx and ly are
#'                  one or zero based.
#' @return a (ly-fy+1) by (lx-fx+1) by (numchanns) array containing the
#' subset of the image from fx to lx, fy to ly inclusive.
#' @note this should mask Voir's version of this function.
#' @export
imsubset <- function( img, fx, fy, lx, ly, one.based=TRUE )
{
    if ( one.based ) {
        fx <- fx-1
        fy <- fy-1
        lx <- lx-1
        ly <- ly-1
    }
    verbosePrintf('fx: %i fy: %i lx: %i ly: %i',fx,fy,lx,ly)
    # for some reason when i put the call directly in the return i only get
    # the first row.
    imgS <- imfastsubset(img,fx,fy,lx,fy) 
    sz <- imgetdims(imgS)
    verbosePrintf('%i x %i x %i',sz[1],sz[2],sz[3])
    return( imgS )
}

#' convert Voir image to array.
#'
#' This is different to Voir's \code{as.array} in that it preserves
#' channels (the former will only return the first channel as an array).
#'
#' @param img      image to be converted
#' @param ...      ignored
#' @return an array of dimensions (nx,ny,nchanns) (if
#' \code{transpose=FALSE}), or (ny,nx,nchanns) (if \code{transpose=TRUE}).
#' @family convert
#' @export
im2array <- function( img, ... )
# convert an image to an array, but handles multi-component images
#  (Voir's as.array will only take first component)
{
    n <- imgetnumcomp(img)
    if ( n == 1 ) return( as.array(img) )
    #if (transpose) {
    #    imgAry <- array(0, dim=c(imgetny(img,1),imgetnx(img,1),n))
    #    for ( i in 1:n ) imgAry[,,i]<-t(as.array(img[[i]]))
    #} else {
    imgAry <- array(0, dim=c(imgetnx(img,1),imgetny(img,1),n))
    for ( i in 1:n ) imgAry[,,i]<-as.array(img[[i]])
    #}
    return(imgAry)
}

#' convert array to Voir image.
#'
#' This is different to Voir's \code{as.image} in that it preserves
#' channels (the former will only return the first channel as an image).
#
#' @param ary        an array of dimensions (ny, nx, nchanns) to 
#'                   be converted.
#' @param outPixType the output pixel type:
#'     \describe{
#'         \item{IM.BINARY}{1 byte, binary pixels, 1 and 0}
#'         \item{IM.CHAR, IM.INT1}{1 byte, char, -127 to 128 }
#'         \item{IM.UINT1}{1 byte, int, 0 to 255}
#'         \item{IM.INT2, IM.SHORT}{2 byte int, -32,767 to 32,768}
#'         \item{IM.UINT2}{2 byte unsigned int, 0 to 65,535}
#'         \item{IM.INT, IM.INT4}{4 byte int, -2,147,483,647 to 2,147,483,648}
#'         \item{IM.UINT4}{4 byte unsigned int, 0 to 4,294,967,295}
#'         \item{IM.INT8}{8 byte int, approx. -9.2e+18 to 9.2e+18}
#'         \item{IM.UINT8}{8 byte unsigned int, 0 to approx. 1.8e+19}
#'         \item{IM.FLOAT}{4 byte float}
#'         \item{IM.DOUBLE}{8 byte double}
#'     }
#' @param ... IGNORED
#' If left NULL, will return an image of type IM.DOUBLE if
#' \code{storage.mode(ary)=="double"}, and IM.INT4 otherwise.
#' @return Voir image from input "ary".
#' @family convert
#' @export
array2im <- function( ary, outPixType=NULL, ... )
{
    if ( is.image(ary) )
        return(ary)
    # imjoin( as.image( ary[,,1]), as.image(ary[,,2]), ... ) 
    dims <- dim( ary )
    if (length(dims)==2) {
        if ( is.null(outPixType) ) {
            return( as.image(ary) )
        } else {
            img <- as.image(ary)
            imsetpixtype(img,IM.ALL,outPixType)
            return( img )
        }
    } else if (length(dims)==3) {
        cmd <- "img <- imjoin("
        for ( i in 1:dims[3] ) {
           cmd <- paste(cmd," as.image(ary[,,",i,"])",ifelse(i==dims[3]," )",","))
        }
        # cat(cmd,"\n")
        eval(parse(text=cmd))
        if ( !is.null(outPixType) ) { imsetpixtype(img,IM.ALL,outPixType) }
        return(img)
    } else {
        stop("error: image should be 2d with at least 1 component ?? (ie img[x,y,channel/z])")
    }
}

#' flips an image about a given axis
#' @param img image array to flip
#' @param type the type of flip: flip horizontally (ie about vertical
#' axis), or vertically (about horizontal axis), or both.
#' @return the input array flipped about the specified dimension
#' @family reshape
#' @export
imflip <- function(img,type=c('horizontal','vertical','both'))
{
    type = match.arg(type,c('horizontal','vertical','both'))
    if ( type == 'horizontal' || type == 'both' )
        img <- immirror(img, IM.ALL, 'MIDV')
    if ( type == 'vertical' || type == 'both' )
        img <- immirror(img, IM.ALL, 'MIDH')
    return(img)
}
