# Common to both packages!
# Contains thermal functions and grid-based functions.

#' flips a 3D *array* about a given axis
#' @param img image array to flip
#' @param type the type of flip: flip horizontally (ie about vertical
#' axis), or vertically (about horizontal axis), or both.
#' @return the input array flipped about the specified dimension
#' @family reshape
imflip.array <- function(img, type=c('horizontal','vertical','both'))
{
    type <- match.arg(type,c('horizontal','vertical','both'))
    sz <- imgetdims(img)
    if ( length(sz)==2 )
        dim(img)<-c(sz[2],sz[1],1)
    if ( type == 'horizontal' || type == 'both' )
        img <- img[,sz[1]:1,]
    if ( type == 'vertical' || type == 'both' )
        img <- img[sz[2]:1,,]
    return(drop(img))
}

#' transposes a 3D array
#' @param img source image to transpose
#' @return matrix of same size as original, transposed for each channel.
#' @family reshape
imtranspose.array <- function(img)
{
    n <- length(dim(img))
    return( aperm(img,c(2,(1:n)[-2])) )
}

#' Determines whether an image is thermal based on its extension.
#'
#' @param fName     the path to the file
#' @return boolean  whether or not the image could be a thermal file
#' @note This just tests that the extension of the file is '.bin', so a
#'  file for which the function returns ``TRUE`` might not actually be a
#'  thermal image file.
#' @export
#' @family thermal
isthermal <- function( fName ) {
    return(getExtension(fName)=='bin')
}

#' Loads a thermal image from a .bin file
#' @param filename the file name of the .bin file
#' @param size the width,height of the image (default 640x480 for the Thermoteknix Miricle)
#' @param max what maximum intensity to scale the image to.
#'                Put NA or 0 or NULL to not scale
#' @param return.what c("array","image"): whether to return an array of the image values,
#'                   or a Voir image (**if you have Voir**).
#' @return a LIST with components:
#'     \describe{
#'        \item{image}{The data/intensities of the thermal image (ie array or Voir image)}
#'        \item{detectorTemperature}{The temperature of the detector in the camera at time of image}.
#'     }
#' @export
#' @family thermal
imloadthermal <- function( filename, size=c(640,480),
                          max=imageUtilities.option('max'), 
                          return.what=c("array","image") )
{
    # open file.
    f <- file(filename,"rb")
    # 640x480 of 2-byte (16bit) ints.
    BIN <- readBin(f,n=640*480,what='int',size=2)
    # 1 of 4-byte (32bit) float
    detectorTemp <- readBin(f,n=1,what=numeric(),size=4)
    close(f)


    imgAry <- matrix(BIN, nrow=640,ncol=480)

    # scale image
    if ( is.numeric(max) && max>0 ) {
        # convert to 0....256 ?
        # a couple values of 0
        MIN <- min( imgAry[imgAry>0] )
        MAX <- base::max( imgAry )

        imgAry <- pmax((imgAry-MIN)/(MAX-MIN) * max,0)
    }
    
    outImg <- imgAry
    if ( tolower(return.what[1])=="image" ) {
        outImg <- as.image(outImg)
    }
    return( list( image=outImg, detectorTemperature=detectorTemp ) )
}
