#' @include common_draw.r
NULL

## NOTE: imview commands for imviewsendcommand are in server/interpreter.cxx
# imviewsendcommand is summarised in server/interpreter.cxx
#            "---- Image management\n"
#            "zoom               : zoom functions, described below:\n"
#            " ... box x y w h   : zoom to specified box \n"
#            " ... factor zf     : specify zoom factor (real number)\n"
#            " ... default zf    : specify default zoom factor\n"
# So to autozoom image to fit my screen, I'd go zoom factor (DESIREDSIZE)/(IMAGESIZE)
# for the moment will do nothing if the current size is already small enough.
# ALSO:
#               "---- Window management\n"
#               "move <X> <Y>       : move main window to new position\n"
#               "size <W> <H>       : change the window dimensions\n"


#' converts a colourmap name to a full path
#' @param cmap name of colourmap (see \code{\link{listColourMaps}} for 
#'             the full list). If not specified, returns all colour maps.
#' @return vector of paths to the colourmap files.
#' @family overlay
#' @keywords internal
getColourmapPath <- function(cmap=NULL)
{
    if ( missing(cmap) ) {
        lut.dir <- system.file('luts',package='imageUtilities')
        cmap <- list.files(lut.dir,pattern='*\\.lut')
    } else {
        cmap <- system.file(sprintf('luts/%s.lut',cmap),
                            package='imageUtilities')
        if ( cmap == "" )
            stopf("Could not find colourmap '%s'",cmap)
    }
    return( cmap )
}

#' lists the available colourmaps for makeOverlay function.
#' @param verbose whether to print the output to screen.
#' @return character vector of colourmap names
#' @family overlay
#' @export
listColourMaps <- function(verbose=TRUE) {
    cmaps <- getFilename(getColourmapPath())
    if ( verbose ) {
        cat("avilable colourmap names:\n")
        cat(cmaps,sep='\n')
    }
    return(cmaps)
}


#' loads a colourmap from its name
#' @inheritParams getColourmapPath
#' @return image of type IMCHAR, dimensions 3x256, with colours in 
#' the rows.
#' @family overlay
#' @export
loadColourMap <- function(cmap)
{
    cmapPath <- getColourmapPath(cmap)
    return(imchar(as.image(t(as.matrix(read.table(cmapPath))))))
}


#' Overlay a binary image onto another image, displaying it within imview.
#'
#' @param img  the image to overlay onto
#' @param ovl  the *binary* image to be overlaid onto img.
#' @param conn the imview connection to display the overlay on (by default
#'              it will make a new one)
#' @param windowTitle The title to use for the overlay (default being the
#'                     variable name of the overlay image...)
#' @param openSaveDialog boolean, default FALSE, whether to open up a "save
#'                       as" dialogue after showing the overlay.
#' @return the imview connection to the overlay window.
#' @note This just displays the overlay on top of the image. If you want
#'  the actual image object consisting of the overlay on the image, use
#'  \code{\link{makeOverlay}}.
#' @family overlay
#' @export
showOverlay <- function( img, ovl, conn = imview.new(), windowTitle=deparse(substitute(ovl)), openSaveDialog=FALSE )
{ 
    # reason for the funny windowTitle is to be able to control the window title :)
    imviewputimage( img, windowTitle , conn ) 
    imview.overlay( ovl ) 
    if ( openSaveDialog ) {
        imviewsendcommand("show save \r\n",conn)
    }
    return(conn)
}  

#' overlays a single-channel binary image onto another.
#' @inheritParams makeOverlay.grid
#' @param cmap name of colour map to use. See \code{\link{listColourMaps}}
#'             for all possible colour maps.
#' @return an image with the overlay on top of it in the specified
#'         transparency and colour, of type IM.CHAR (IM.UINT1).
#' @note if you want to only *view* the overlay and don't care about having
#' the image object, use \code{\link{showOverlay}}.
#' @family overlay
#' @export
makeOverlay <- function( img, overlay, transparency=0.5, col=c(255,0,0),
                        cmap="regions" )
{ 
    cmaps <- listColourMaps(verbose=FALSE)
    cmap <- match.arg(cmap,cmaps)
    if ( is.character(col) ) col <- as.vector(col2rgb(col))
    if ( !is.vector(col) ) col <- as.vector(col)

    # look up colour map.
    cmap <- loadColourMap(cmap)
    # find closest colour
    idx  <- which.min( apply( as.array(cmap),2,function(x) return(sum(abs(x-col)))  ) )-1 #-1 because 1-/0-based
    if ( idx == 0 ) { idx <- min(imgetny(cmap,1),255) } # wrap around.
    # scale img to be 0 and that colour
    ovl <- imcmul( imcmin( overlay, 1 ), idx, IM.INT4 )  # the only overlay type the function accepts

    return(imoverlaytransparent( imchar(img), ovl, cmap, transparency ))
} # makeOverlay

#' retrieves points interactively from an image.
#' 
#' @param imageOrPath the image to be selected from
#' @param customInstructions custom instructions to be printed to the
#' screen
#' @param killWindow  whether to kill the imview window once points are
#' picked.
#' @return a LIST with the following elements:
#' \describe{
#'     \item{points}{an nx9 array, n being the number of points selected.
#'     It has columns "X", "Y", "Z" giving (0-based) coordinates of the
#'     points selected, columns "x", "y", "z", being the same as the
#'     upper-case versions, and columns "R", "G", "B", with the intensities
#'     in each channel of the image at the selected points.}
#'     \item{ngroups}{the number of groups selected (1)}
#'     \item{type}{The type of the points ('colour')}
#'     \item{nplanes}{The number of channels in the points (equal to the
#'     number of channels in the image}
#' }
#' @seealso \code{\link{pointPicker}}, which uses grid instead of imview and restricts the
#' number of points to be chosen.
#' @export
getImagePoints <- function( imageOrPath, customInstructions="Then click on the points you wish to select.\n", killWindow=TRUE )
{
##### ----- 0. Error/input checking ----- #####
    # see if the input is an img or path
    img <- imload( imageOrPath )

    if ( is.null(img) || !is.image(img) ) {
        stop("The input was neither an image nor a file path to one, or the image is corrupted.\n")
    }

##### ----- 1. Get Points ----- #####
    pts <- NULL

  # Display image, ask user to make a pointfile, wait for response.
    IVcon <- imview.new() # imview window for picsj
    imview(img, im.connection=IVcon)
   	cat("Select points by SHIFT-CLICKing on them.\n")
    cat(customInstructions)
    cat("When finished, press ENTER")
  	scan(file="", quiet=TRUE, what = character(0), nlines=1)
 
##### ----- 2. Return Points ----- #####
  # Load the pointfile, return points.
    pts <- imview.getpointfile(im.connection = IVcon)  
    if ( killWindow ) {
        imview.kill( im.connection=IVcon )
    }
    nms <- names(pts)
    nms[nms==""] <- "points"
    names(pts) <- nms

# Does imview put this here?
    if ( file.exists('pointfile.bak') ) file.remove('pointfile.bak')
    return(pts)
} 

#' plot points onto an image with lines around them.
#' 
#' @param img    the image to draw on to.
#' @param ipts   an nx2 matrix with points to be drawn on the image. Column
#'               names "X" and "Y".
#' @param radius the radius of points to draw (pixels).
#' @param lines  boolean,whether to draw lines in between the points
#' @param transparency The transparency of the points. 0 for opaque, 1 for
#'         transparent.
#' @param col     The colour to draw the overlay in.
#'                Either a string of a colour recognised by R, or
#'                 a numeric vector \code{c(R,G,B)} with red/green/blue
#'                 values the desired colour, from 0 to 255.
#' @return image with the lines and points drawn on.
#' @family draw
#' @export
plotPoints <- function( img, ipts, radius=min(imgetnx(img,1),imgetny(img,1))/100, lines=FALSE, transparency=0, col=c(255,0,0) ) 
{
    if ( is.character(col) ) col <- as.vector(col2rgb(col))
    if ( !is.vector(col) ) col <- as.vector(col)

    Ary <- array(0,dim=c(imgetnx(img,1),imgetny(img,1)))
    pts <- ipts
    if ( nrow(ipts)==2 && ncol(ipts)!=2 ) 
        pts <- t(ipts)
    if ( !is.null(colnames(pts)) )
        pts <- pts[,c("X","Y")]
    else
        colnames(pts) <- c("X","Y")

    pts <- sortPointsRectangle( pts )[,c("X","Y")]
    Ary[pts] <- 1
    msk <- as.image(Ary)

    # FIRST: reorganise points so they go clockwise or anticlockwise, not cross over.

    if ( lines ) {
        pts <- pts + 1 #stoopid but imdrawline is 1-based whereas all other im functions are 0-based.
        for ( i in 1:nrow(pts) ) {
            j <- ifelse( i==nrow(pts), 1, i+1 )
            imdrawline( msk, 1, pts[i,"X"],pts[i,"Y"], pts[j,"X"],pts[j,"Y"],255 )
        }
    }
    msk <- imbinary(msk)
    if ( radius>1 ) 
        msk <- !imbderode(!msk,radius)
  #      msk <- imbddilate( msk, radius )

    ovl <- makeOverlay(img, msk,col=col,transparency=transparency)

    return(ovl)
}

