# for thermal stuffs.
#' @include common_imaging_utilities.r
NULL
# *************** I/O ****************** #
#' loads an image from a file path.
#'
#' @param  imageOrPath the path to the image to load.
#' @param  max what intensity to scale the output images to (default 255).
#' @return a Voir image.
#' @note supported formats: jpeg, tiff (require Voir), bin (ie thermal),
#'       png (requires library(png))
#'
#' Also, masks Voir::imload (!) which is for z-image.
#' @family io
#' @export
imload <- function( imageOrPath,
                   max=imageUtilities.options('max') ) 
#  loads an image from a filepath. if an image is supplied then it just returns that.
{
    img <- NULL
    # see if the input is an img or path
    if ( is.image(imageOrPath) ) {
        img <- imageOrPath
    } else if ( is.character(imageOrPath) ) {
        # Get Image & error check.
        ext <- getExtension( imageOrPath )
        supportedExts <- c('png','jpeg','jpg','tiff','tif','bin')
        tryCatch({
            ext <- match.arg(ext, supportedExts)
        }, error = function(e) {stopf('imload: unsupported/unknown extension %s\n',ext) })

        if ( max==255 )
            f <- identity
        else
            f <- function(x) x/255*max
 
        if ( any(grep( "jpe?g",ext )) ) {
	        img <- f(imloadjpeg(imageOrPath))
        } else if ( any(grep( "tiff?",ext )) ) {
    	    img <- f(imloadtiff(imageOrPath))
        } else if ( any(grep("bin",ext)) ) {
            img <- imloadthermal(imageOrPath,return.what='image',max=max)$image
        } else if ( ext == "png" ) {
            library(png)
            #if ( require(png) )
            img <- as.image(readPNG(imageOrPath)*max)
        }
    }    
    if ( is.null(img) || !is.image(img) ) {
        stop("Unsupported image format, or the image is corrupted.\n")
    }

    return(img)
}

#' saves an image (tiff, jpeg, png, bmp)
#' @param img     the image to save
#' @param name    the file name to save as
#' @param comment save this as the image's comment (if JPEG or TIFF)
#' @param qual    quality of the file (if JPEG, default 100)
#' @param max     maximum intensity of input image.
#' @note bmp and png use the base \code{\link[grDevices]{png}} and
#' \code{\link[grDevices]{bmp}} functions in combination with
#' (internal to this package) \code{\link{imsave.grid}}, which needs
#' package 'grid'.
#' Alternatively, the png option can use the png package.
#' @seealso \code{\link[png]{writePNG}}
#' @family io
#' @export
imsave <- function( img, name, comment=NULL, qual=100,
                   max=imageUtilities.options('max'))
{
    ext <- getExtension( name )
    supportedExts <- c('png','jpeg','jpg','tiff','tif','bmp')
    tryCatch({
        ext <- match.arg(ext, supportedExts)
    }, error = function(e) {stopf('imsave: unsupported/unknown extension %s\n',ext) })
    outc <- -1 
    
    if ( max == 255 )
        f <- identity
    else
        f <- function(x) x/max*255

    if ( any(grep( "jpe?g",ext )) ) {
        if ( is.null(comment) || length(comment)==0 || comment=="" ||
            !exists('imsavejpegwithcomment',mode='function') ) {
            outc <- imsavejpeg(f(img), name, qual)
        } else {
            outc <- imsavejpegwithcomment(f(img), name, qual, comment)
        }
    } else if ( any(grep( "tiff?",ext )) ) {
        if ( is.null(comment) || length(comment)==0 || comment=="" ||
            !exists('imsavetiffwithcomment',mode='function') ) {
            outc <- imsavetiff(f(img), name)
        } else {
            outc <- imsavetiffwithcomment(f(img), name, comment)
        }
    } else if (ext == 'png' && require(png)) {
        writePNG(im2array(img)/max,name)
    } else if ( ext %in% c('png','bmp') && require(grid) ) {
        imsave.grid(im2array(img),name,FUN=ext,max=max)
    } else {
        stop("imsave:unrecognised extension")
    }
    return(outc)
}

#' sets an image's comment (JPEG and TIFF only)
#' @param name    file name of image you want to set the comment of
#' @param comment the comment to add.
#' @note this sets the 'ImageDescription' tiff tag of a tiff file, and the
#' 'comment' tag of a jpeg file.
#' @export
imsetcomment <- function( name, comment ) 
{
    img <- imload(name)
    return(imsave( img, name, comment=comment ))
    rm(img)
}

