#' filters a binary image according to statistics of the regions in it.
#'
#' @param img          Image to be filtered.
#' @param prop         String, one of "area", "perimeter", "rowcentre",
#'                     "columncentre", "minellipse", "majellipse", 
#'                     "angleellipse", "MERtop", "MERbottom", "MERleft" 
#'                     or "MERright".
#' @param THRESH       optional. The threshold at which to filter the
#'                     objects. If negative, the MEAN of the property will
#'                     be used.
#' @param imstat       optional. If supplied, these statistics will be used
#'                     rather than recalculating them.
#'                     Should be the output of \code{\link[Voir]{imstats}}.
#' @param foreground   the value of the foreground objects: 1 or 0.
#' @param connectivity connectivity with which to calculate statistics: 4
#'                     or 8.
#' @param op           string of a \code{op(a$prop,THRESH)} returning either 
#'                     TRUE or FALSE. Regions in the image for which
#'                     \code{op(blob$prop,THRESH)} returns TRUE, are
#'                     retained in the output.
#'                     Default ">", ie retains regions with \code{prop >
#'                     THRESH}.
#' @param return.stats Boolean, whether to return the calculated statistics
#'                     from \code{\link[Voir]{imstats}}.
#'
#' @return if \code{return.stats} is FALSE, a binary image that is TRUE for
#' objects for which \code{op(a$prop,THRESH)} is TRUE.
#'
#' If \code{return.stats} is TRUE, a list:
#'     \describe{
#'        \item{stats}{A matrix containing the statistics of all the
#'        objects remaining in the image (column names "area", "perimeter",
#'        ...)}
#'         \item{image}{The binary image containing all the regions that
#'         satisfied \code{op(blob$prop,THRESH)==TRUE}.}
#'         \item{threshold}{The threshold used (helpful if the input THRESH
#'         was -1 and had to be calculated.}
#'      }
#' @family morphology
#' @seealso \code{\link[Voir]{imstats}}
#' @export
filterByProperty <- function( img, prop, THRESH=-1, imstat=NULL, foreground=1,
                         connectivity=4, return.stats=FALSE, op=">" )
{
    if ( missing(imstat) ) {
        # what's the huge 6e6 one !! I reckon that's the ground! maybe I want foreground=0
        Simg <- imstats(img,foreground=foreground,connectivity=connectivity)
    } else {
        Simg <- imstat   
    }
    S <- imobjstats(Simg)
    if ( !(prop %in% colnames(S)) )
        stopf("filterByProperty: unknown property '%s'",prop)

    BFUNC <- match.fun(op)

    if (THRESH <0) THRESH <- mean(S[,prop])
    Sf <- as.matrix(S[BFUNC(S[,prop],THRESH),]) # o'wise problems when only one row.
    SimgF <- NULL
    nx <- imgetnx(img,1)
    ny <- imgetny(img,1)
    SAry <- as.array(Simg)
    if ( nrow(Sf) == 1 || ncol(Sf)==1 ) 
        Sf <- t(Sf)
        
    SAryF <- array(SAry %in% Sf[,"number"],dim=dim(SAry))  # lookup %in% as ?match
    SimgF <- as.image(SAryF)
    SimgF <- imbinary(SimgF)
    # cat(sum(Sf[,"area"])/(nx*ny),'\n')

    if ( return.stats ) {
        return(list(stats=Sf,image=SimgF,threshold=THRESH))
    } else {
        return(SimgF)
    }
}

#' filters a binary image, removing all objects with area smaller than
#' THRESH.
#'
#' @inheritParams filterByProperty
#' @return if \code{return.stats} is FALSE, a binary image that is TRUE for
#' objects for which \code{op(a$prop,THRESH)} is TRUE.
#'
#' If \code{return.stats} is TRUE, a list:
#'     \describe{
#'        \item{stats}{A matrix containing the statistics of all the
#'        objects remaining in the image (column names "area", "perimeter",
#'        ...)}
#'         \item{image}{The binary image containing all the regions that
#'         satisfied \code{op(blob$prop,THRESH)==TRUE}.}
#'         \item{threshold}{The threshold used (helpful if the input THRESH
#'         was -1 and had to be calculated.}
#'      }
#' @note this is just a wrapper around \code{\link{filterByProperty}} with
#' \code{prop="area"}.
#' @family morphology
#' @seealso \code{\link[Voir]{imstats}}
#' @export
filterBySize <- function( img, THRESH=-1, imstat=NULL, foreground=1,
                         connectivity=4, return.stats=FALSE, op=">" ) 
{
    return( filterByProperty( img, "area", THRESH=THRESH, imstat=imstat,
                         foreground=foreground, connectivity=connectivity,
                         return.stats=return.stats, op=op ) )
}
