#' displays an image within imview with automagic window size
#'
#' This function displays an image within imview, automatically reizing the
#' window such that it fills at most \code{fractionOfScreen} of the screen.
#'
#' This can be handy when dealing with big images that typically start up
#' (in imview) at their full size, bein larger than the screen.
#'
#' @param img the image to display
#' @param fractionOfScreen between 0 and 1, the fraction of the user's
#' @param ... UNUSED (for compatibility with non-Voir version of the
#' package)
#' screen to use up with the window. The window will be resized such that
#' the *maximum* dimension takes up
#' \code{fractionOfScreen*screenResolution} for that dimension.
#' @return the imview connection to the new window
#'
#' @note This function will only work if it is able to determine your
#' screen resolution. See \code{\link{getScreenResolution}} 
#' for that; on unix it uses \code{xdpyinfo}; on windows it uses
#' \code{wmic}.
#'
#' If it is unable to determine the screen size, a default maximum
#' dimension of 640 pixels will be used.
#' @export
imdisp <- function( img, fractionOfScreen=.5, ... )
{
    nx  <- imgetnx(img,1)
    ny  <- imgetny(img,1)
    # target size
    res <- getScreenResolution(quietly=TRUE)
    if ( !is.null(res) )
        szt <- res*fractionOfScreen
    else # use default.
        szt <- imageUtilities.option('default.screen.size')

    con <- imview.new(img)    
    if ( nx > szt[1] || nx > szt[2] ) {
        zoomF <- min( szt/c(nx,ny) )
        #cat('zoomF:',zoomF,'\n')
        imviewsendcommand(sprintf('size %i %i\r\n', szt[1], szt[2]))
        imviewsendcommand(sprintf('zoom factor %f\r\n',zoomF))
    }
    return(con)
}



# LINUX ONLY: NEED SCREEN SIZE
# mosaic all the results?
# fractionOfScreen <= 0: don't scale at all.
# trusts that all images are of the same size.
#' tries to intelligently mosaic images with respect to screen size (may
#' not work)
#'
#' @param imgList vector of image names to mosaic
#' @param dim     (nrow,ncol) dimension of mosaic in images per dimension.
#'                Leave NULL to automagically calculate
#' @param screen.fraction The fraction of the screen the resulting mosaic
#' should take up
#' @param minimum.fit boolean, resize so that the largest dimension fits on
#' \code{screen.faction*screensize}.
#' @return image containing all the input images mosaiced together (black
#' space if the number of input images doesn't perfectly fit in a
#' rectangle).
#' @note linux only as this requires a screen size.
#' 
#' This attempts to determine the optimal number of images to put in the
#' rows and columns of the mosaic based off screen size and image size
#' (assumes all input images are the same size).
#' 
#' It hasn't been tested in forever so may not work.
#' @export
makeMosaic <- function( imgList, dim=c(-1,-1), screen.fraction=1, minimum.fit=TRUE ) 
{
    N <- length(imgList)
    mosaicImg <- NULL

    if ( N == 1 ) {
        if ( screen.fraction <= 0 )
            imview.new( imload(imgList[1]) )
        else
            imdisp( imload(imgList[1]), screen.fraction )
    } else {
        sz <- getScreenResolution()
        # get screen size (does twinview so total size..).     
        img <- imload( imgList[1] )
        nxi <- imgetnx(img,1)
        nyi <- imgetny(img,1)
        imgtype <- imgetimgtype(img)
        pixtype <- imgetpixtype(img,1)

        if ( !any(dim>0) ) {
            # this is kind of faulty: i suppose I want to minimise the 
            # number of blank images while still conforming to the aspect
            # ratio of the screen, min. blank images should take precedence!

            # automagically determine the rows/cols.
            # will try to best fit screen size.
            # so ncol*nxi / nrow*nyi == screenx / screeny
            # => ncol/nrow == screenx/screeny * nyi/nxi
            aspRatio_weighted <- sz[1]/sz[2] * nyi/nxi
            # have :
            # ncol / nrow =  aspR   ==> ncol == nrow*aspR
            # ncol * nrow >= N
            # SO:
            # aspR * nrow^2 >= N
            # nrow^2 >= N/aspR         [all values +ve here]
            # nrow   >= sqrt(N/aspR)   [ as nrow >= 1 <=> nrow^2 >= 1 ]
            # so take next highest int.
            nrows <- ceiling( sqrt(N/aspRatio_weighted) )
            ncols <- ceiling( N/nrows )
        } else {
            nrows <- ifelse( dim[1]>0, dim[1], ceiling(N/dim[2]) )
            ncols <- ifelse( dim[2]>0, dim[2], ceiling(N/dim[1]) )
        } 

        # construct the rows.
        # will subsample each image individually then join
        # as opposed to joining then subsampling.

        szout <- c( ncols*nxi, nrows*nyi )
        # minimum.fit: make smaller so that the largest dimension fits
        #        if FALSE: make so that the smallest dimension fits.
        sFact <- ifelse( minimum.fit, max(szout/(screen.fraction*sz)), min(szout/(screen.fraction*sz)) )
        sFact <- ifelse( minimum.fit, ceiling(sFact), floor(sFact) )

        # SLOW :: JOINS ONE ATRUE A TIME INSTEAD OF TAKING ADVANTAGE OF THE ... PARAMETER 
        #  o'wise may need to load/subsample say 20 images before joining!
        n <- 1
        # cat(sprintf('dimensions: %ix%i\n',nrows,ncols))

        for ( i in 1:nrows ) {
            imrow <- NULL
            # construct the rows.
            for ( j in 1:ncols ) {
                if ( n > N )  {
                    # img <- imconstructrgb(IM.UINT1,nxi,nyi,0,0) # for some reason imview doesn't like that.
                    img <- imchar(as.image(array(0,dim=c(nxi,nyi))))
                    if ( imgtype==IM.RGB )
                        img <- imjoin(img,img,img)
                } else  {
                    img <- imload( imgList[n] )
                } 
                if ( imgetpixtype(img,1) == IM.BINARY ) img <- imchar(img)
                n   <- n + 1
                # only bother downsampling if you have to make images smaller.
                if ( sFact > 1 )
                    img <- imdownsample( img, sFact, sFact, 1 )

                imrow <- imweld( imrow, img, mode="across" )
            }
            mosaicImg <- imweld( mosaicImg, imrow, mode="down" )
        }
        imsetpixtype( mosaicImg, IM.ALL, pixtype )
        imsetimgtype( mosaicImg, imgtype )
    } # >1 image
   
    return( mosaicImg )
}
